#pragma rtGlobals=3		// Use modern global access method.

//Update 20100418: added MorphROI
//Update 20140516: Added 3D support
//Update 20160708: Roistats for Timeseries (with 2D ROI)
//Update 20170319: ROIStats for Timeseries with 3D ROI

// MultiROI(binarymask, targetwave) numbers regions of interest (ROIs) from the binary
// ROI mask sourcewave (pixels in the ROI have a value of 0, those outside 1) and generates
// a MultiROi ROI wave with the name targetwave. In this wave, ROIs will have increasing negative
// numbers starting with -1 near the origin. ROIs are numbered line-by-line, with rows increasing 
// at the end of each line and layers increasing after that, if binarymask has 3 dimensions. 
// Furthermore, if pixels "touch" on an edge, the will be considered to be the
// same ROI. This can be changed by commenting out the conditional statements where neither x2 nor
// y2 nor z2 is 0.
//If roicount > 0 the first ROI will have the value roicount.
//If closing > 0 image closure will be pwerformed on binarymask. This can have unexpected results if
//binarymask is not actually binary.
// MorphROI(ROI, type) performs morphological operations on the MultiROIwave ROI, depending on type:
// 0=Erosion, 1=Dilation, 2=Closing, 3=Opening;
//MultiROIStats(image, ROI, [m]) calculates statistics such as min, max, avg, number of pixels (using imagestats) 
//on each ROI. m defines the moments passed to imagestats
//PixelROIs(ROIWave) calculates a new ROI wave, based on ROIWave, in which each ROI pixel gets a unique identifier.

Function MultiROI(binarymask, targetwave, [roicount, closing, overwrite])
	wave binarymask
	string targetwave
	variable ROIcount, closing, overwrite
	
	variable nDims, result
	
	if(paramisdefault(ROICount))
		roicount = 0
	endif
	
	if(ParamIsDefault(closing))
		closing = 0
	endif
	
	if(ParamIsDefault(overwrite))
		overwrite = 0
	endif
	
	nDims = WaveDims(binarymask)
	
	switch(nDims)
	
		case 2:	//2D
			result = MultiROI2D(binarymask, targetwave, roicount=roicount, closing=closing, overwrite=overwrite)
		break
	
		case 3:	//3D
			result = MultiROI3D(binarymask, targetwave, roicount=roicount, closing=closing, overwrite=overwrite)
		break
		
		default:
			result = -1
		break
	
	endswitch
	
	return result
end


///////////////////////////////////////////////////////////////////////////////

Function MultiROI2D(binarymask, targetwave, [roicount, closing, overwrite])
	wave binarymask
	string targetwave
	variable ROIcount, closing, overwrite
	
	if(paramisdefault(ROICount))
		roicount = 0
	endif
	
	if(ParamIsDefault(closing))
		closing = 0
	endif
	
	if(ParamIsDefault(overwrite))
		overwrite = 0
	endif
	
	
	
	variable xdim, ydim, xcount = 0, ycount = 0, x2, y2, condition, minx, condition2, maxx, type, pixelnumber = 0
	variable x3,y3, pxtrack
	string info
	
	info =waveinfo(binarymask, 0)
	type =  NumberByKey("NUMTYPE", info)
	
	
	if(closing>0)
		duplicate/o/free binarymask, locmask
		redimension/b/u locmask
		ImageMorphology Closing locmask	
		Wave M_ImageMorph
		duplicate/o/free M_ImageMorph, sourcewave
	else
		wave sourcewave=binarymask
	endif
	
	
	duplicate /o/free sourcewave, mrcalcwave
	xdim = dimsize(sourcewave, 0)
	ydim = dimsize(sourcewave, 1)
	make /o/free/n=(xdim*ydim,2) pointstore
	
	
	if (type > 5)
		redimension /d mrcalcwave
	endif
	FastOP mrcalcwave = 1
	
	ycount = 0
	do	//ycount
	xcount = 0
		do //xcount
		pixelnumber = 0
			if ((sourcewave[xcount][ycount] < 1) && (mrcalcwave[xcount][ycount]==1))
				pixelnumber +=1
				ROIcount +=1
				pxtrack = 0
				mrcalcwave[xcount][ycount]= -roicount
				pointstore[pixelnumber - 1][0] = xcount
				pointstore[pixelnumber - 1][1] = ycount
				x3 = xcount
				y3 = ycount
				
				do //pxtrack
					x2 = -1
					y2 = -1
					
					if((x3+x2 > 0) && (y3+y2 > 0) && (x3+x2 < xdim) && (y3+y2 < ydim))	
						if ((sourcewave[x3+x2][y3+y2] < 1) && (mrcalcwave[x3+x2][y3+y2]==1))
							pixelnumber +=1
							mrcalcwave[x3+x2][y3+y2]= -roicount
							pointstore[pixelnumber - 1][0] = x3+x2
							pointstore[pixelnumber - 1][1] = y3+y2
						endif
					endif
					
					
					x2 = 0
					y2 = -1
					if((x3+x2 > 0) && (y3+y2 > 0) && (x3+x2 < xdim) && (y3+y2 < ydim))		
						if ((sourcewave[x3+x2][y3+y2] < 1) && (mrcalcwave[x3+x2][y3+y2]==1))
							pixelnumber +=1
							mrcalcwave[x3+x2][y3+y2]= -roicount
							pointstore[pixelnumber - 1][0] = x3+x2
							pointstore[pixelnumber - 1][1] = y3+y2
						endif
					endif
						
						x2 = 1
						y2 = -1
					if((x3+x2 > 0) && (y3+y2 > 0) && (x3+x2 < xdim) && (y3+y2 < ydim))	
						if ((sourcewave[x3+x2][y3+y2] < 1) && (mrcalcwave[x3+x2][y3+y2]==1))
							pixelnumber +=1
							mrcalcwave[x3+x2][y3+y2]= -roicount
							pointstore[pixelnumber - 1][0] = x3+x2
							pointstore[pixelnumber - 1][1] = y3+y2
						endif
					endif
						
						x2 = -1
						y2 = 0
					if((x3+x2 > 0) && (y3+y2 > 0) && (x3+x2 < xdim) && (y3+y2 < ydim))	
						if ((sourcewave[x3+x2][y3+y2] < 1) && (mrcalcwave[x3+x2][y3+y2]==1))
							pixelnumber +=1
							mrcalcwave[x3+x2][y3+y2]= -roicount
							pointstore[pixelnumber - 1][0] = x3+x2
							pointstore[pixelnumber - 1][1] = y3+y2
						endif
					endif
						
						x2 = 1
						y2 = 0
					if((x3+x2 > 0) && (y3+y2 > 0) && (x3+x2 < xdim) && (y3+y2 < ydim))	
						if ((sourcewave[x3+x2][y3+y2] < 1) && (mrcalcwave[x3+x2][y3+y2]==1))
							pixelnumber +=1
							mrcalcwave[x3+x2][y3+y2]= -roicount
							pointstore[pixelnumber - 1][0] = x3+x2
							pointstore[pixelnumber - 1][1] = y3+y2
						endif
					endif
						
						x2 = -1
						y2 = 1
					if((x3+x2 > 0) && (y3+y2 > 0) && (x3+x2 < xdim) && (y3+y2 < ydim))	
						if ((sourcewave[x3+x2][y3+y2] < 1) && (mrcalcwave[x3+x2][y3+y2]==1))
							pixelnumber +=1
							mrcalcwave[x3+x2][y3+y2]= -roicount
							pointstore[pixelnumber - 1][0] = x3+x2
							pointstore[pixelnumber - 1][1] = y3+y2
						endif
					endif
						
						x2 = 0
						y2 = 1
					if((x3+x2 > 0) && (y3+y2 > 0) && (x3+x2 < xdim) && (y3+y2 < ydim))	
						if ((sourcewave[x3+x2][y3+y2] < 1) && (mrcalcwave[x3+x2][y3+y2]==1))
							pixelnumber +=1
							mrcalcwave[x3+x2][y3+y2]= -roicount
							pointstore[pixelnumber - 1][0] = x3+x2
							pointstore[pixelnumber - 1][1] = y3+y2
						endif
					endif
						
						x2 = 1
						y2 = 1
					if((x3+x2 > 0) && (y3+y2 > 0) && (x3+x2 < xdim) && (y3+y2 < ydim))	
						if ((sourcewave[x3+x2][y3+y2] < 1) && (mrcalcwave[x3+x2][y3+y2]==1))
							pixelnumber +=1
							mrcalcwave[x3+x2][y3+y2]= -roicount
							pointstore[pixelnumber - 1][0] = x3+x2
							pointstore[pixelnumber - 1][1] = y3+y2
						endif
					endif
					
					
				pxtrack +=1
				x3 = pointstore[pxtrack][0]
				y3 = pointstore[pxtrack][1]
				while(pxtrack <= pixelnumber)
				
			endif
		xcount +=1
		while(xcount<xdim)
	ycount +=1
	while (ycount<ydim)
	
	
	
	if(overwrite>0)
		duplicate/o mrcalcwave, binarymask
	else
		duplicate /o mrcalcwave, $targetwave
	endif
	
	return ROIcount
end

////////////////////////////////////////////



Function MultiROI3D(binarymask, targetwave, [roicount, closing, overwrite])
	wave binarymask
	string targetwave
	variable ROIcount, closing, overwrite
	
	if(paramisdefault(ROICount))
		roicount = 0
	endif
	
	if(ParamIsDefault(closing))
		closing = 0
	endif
	
	if(ParamIsDefault(overwrite))
		overwrite = 0
	endif
	
	
	variable xdim, ydim, zdim, xcount, ycount, zcount, x2, y2, z2, type, pixelnumber = 0
	variable x3,y3,z3, pxtrack
	string info
	
	if(closing>0)
		duplicate/o/free binarymask, locmask
		redimension/b/u locmask
		ImageMorphology Closing locmask	
		Wave M_ImageMorph
		duplicate/o/free M_ImageMorph, sourcewave
	else
		wave sourcewave=binarymask
	endif
	
	
	info =waveinfo(binarymask, 0)
	type =  NumberByKey("NUMTYPE", info)
	

	wave sourcewave=binarymask
	
	duplicate /o/free sourcewave, mrcalcwave
	xdim = dimsize(sourcewave, 0)
	ydim = dimsize(sourcewave, 1)
	zdim = dimsize(sourcewave, 2)
	
	
	make /o/free/n=(xdim*ydim*zdim,3) pointstore		
	
	
	if (type > 5)
		redimension /d mrcalcwave
	endif
	//FastOP 
	mrcalcwave = 1
	
	
	zcount=0
	do  //zcount
		ycount = 0
		do	//ycount
		xcount = 0
			do //xcount
			pixelnumber = 0
				if ((sourcewave[xcount][ycount][zcount] < 1) && (mrcalcwave[xcount][ycount][zcount]==1))
					pixelnumber +=1
					ROIcount +=1
					pxtrack = 0
					mrcalcwave[xcount][ycount][zcount]= -roicount
					pointstore[pixelnumber - 1][0] = xcount
					pointstore[pixelnumber - 1][1] = ycount
					pointstore[pixelnumber - 1][2] = zcount
					x3 = xcount
					y3 = ycount
					z3= zcount
					
					do //pxtrack
						//layer +0
						z2 = 0
						
						x2 = -1
						y2 = -1
						
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))						
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
											
						
						x2 = 0
						y2 = -1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))		
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
						
						x2 = 1
						y2 = -1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))		
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
						
								///repeat 2x if from above
						
						x2 = -1
						y2 = 0
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
						
						x2 = 1
						y2 = 0
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
						
						x2 = -1
						y2 = 1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
						
						x2 = 0
						y2 = 1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
						
						x2 = 1
						y2 = 1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
						
						
						
						
							//layer +1
							z2 = 1
							
							x2 = 0
							y2 = 0		
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))				
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
										
							x2 = -1
							y2 = -1	
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))					
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = 0
							y2 = -1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = 1
							y2 = -1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = -1
							y2 = 0
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = 1
							y2 = 0
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = -1
							y2 = 1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = 0
							y2 = 1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = 1
							y2 = 1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif



						//layer -1
						z2 = -1
							
							x2 = 0
							y2 = 0		
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))				
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
										
							x2 = -1
							y2 = -1	
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))					
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = 0
							y2 = -1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = 1
							y2 = -1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = -1
							y2 = 0
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = 1
							y2 = 0
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = -1
							y2 = 1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = 0
							y2 = 1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif
							
							x2 = 1
							y2 = 1
						if((x3+x2 > 0) && (y3+y2 > 0) && (z3+z2>0) && (x3+x2 < xdim) && (y3+y2 < ydim) && (z3+z2<zdim))
							if ((sourcewave[x3+x2][y3+y2][z3+z2] < 1) && (mrcalcwave[x3+x2][y3+y2][z3+z2]==1))
								pixelnumber +=1
								mrcalcwave[x3+x2][y3+y2][z3+z2]= -roicount
								pointstore[pixelnumber - 1][0] = x3+x2
								pointstore[pixelnumber - 1][1] = y3+y2
								pointstore[pixelnumber - 1][2] = z3+z2
							endif
						endif

						
					pxtrack +=1
					x3 = pointstore[pxtrack][0]
					y3 = pointstore[pxtrack][1]
					z3 = pointstore[pxtrack][2]
					while(pxtrack <= pixelnumber)
					
				endif
			xcount +=1
			while(xcount<xdim)
		ycount +=1
		while (ycount<ydim)
	zcount+=1
	while(zcount<zdim)
	
	
	
	if(overwrite>0)
		duplicate/o mrcalcwave, binarymask
	else
		duplicate /o mrcalcwave, $targetwave
	endif
	
	return ROIcount
end




/////////////////////////////////////

Function MultiROIbyLayer(sourcewave, targetwave)
	wave sourcewave
	string targetwave
	
	
	Variable zDim = DimSize(sourcewave, 2), ii, ROInumber = 0, counter
	
	if(zDim < 1)
		zDim = 1
	endif
	
	duplicate/o/free sourcewave MRW
	
	for(ii=0;ii<zDim;ii+=1)
		duplicate/o/free/r=[0,*][0,*][ii] sourcewave frame
		
		counter=ROInumber
		ROInumber=multiroi2D(frame,"W_multiROI", roicount=counter)
		wave W_multiROI
	
		MultiThread MRW[][][ii]=W_multiROI[p][q]
	
	endfor
	

	duplicate/o MRW $targetwave
	
	killwaves W_multiROI
	return ROInumber
End


/////////////////////////////////////

Function MultiROILoop(sourcewave, targetwave)
	wave sourcewave
	string targetwave
	
	
	Variable zDim = DimSize(sourcewave, 2), ii, ROInumber = 0, counter
	
	if(zDim < 1)
		zDim = 1
	endif
	
	duplicate/o/free sourcewave MRW
	
	for(ii=0;ii<zDim;ii+=1)
		duplicate/o/free/r=[0,*][0,*][ii] sourcewave frame
		
		counter=ROInumber
		ROInumber=multiroi2D(frame,"W_multiROI", roicount=0)
		wave W_multiROI
	
		MultiThread MRW[][][ii]=W_multiROI[p][q]
	
	endfor
	

	duplicate/o MRW $targetwave
	
	killwaves W_multiROI
	return ROInumber
End

/////////////////////////////////////

Function/Wave MultiROIStats(image, ROI, [m])
	Wave Image, ROI
	Variable m
	
	Variable mismatch

	Mismatch = abs(WaveDims(image) - WaveDims(ROI)) + abs(DimSize(image,0) - DimSize(ROI, 0)) + abs(DimSize(image, 1) - DimSize(ROI, 1)) + abs(DimSize(image,2) - DimSize(ROI, 2))
	
	If(mismatch)			//just checking...
		DoAlert 0, "Image/ROI mismatch"
		Make/o/n=(1,4) ROIStats = NaN		
		Return ROIStats
	endif
	
	
	If(ParamIsDefault(m))
		m=1
	elseif(!((m == 1) || (m == 2)))
		m = 2
	endif
	
	Variable ROInumber, ii
	
	ROInumber = -WaveMin(ROI)
	
	if(m==1)
		Make/o/n=(ROInumber,4) ROIStats
		SetDimLabel 1, 0, Avg, ROIStats
		SetDimLabel 1, 1, Min, ROIStats
		SetDimLabel 1, 2, Max, ROIStats
		SetDimLabel 1, 3, npnts, ROIStats
	else
		Make/o/n=(ROInumber,9) ROIStats
		SetDimLabel 1, 0, Avg, ROIStats
		SetDimLabel 1, 1, Min, ROIStats
		SetDimLabel 1, 2, Max, ROIStats
		SetDimLabel 1, 3, npnts, ROIStats
		
		SetDimLabel 1, 4, sdev, ROIStats
		SetDimLabel 1, 5, rms, ROIStats
		SetDimLabel 1, 6, skew, ROIStats
		SetDimLabel 1, 7, kurt, ROIStats
		SetDimLabel 1, 8, adev, ROIStats	
	endif
	
	Duplicate/o/free ROI ROImask
	ROImask = 1
	Redimension /b/u ROIMask
	
	For(ii=0;ii<ROINumber;ii+=1)
	
//		ROIMask = selectnumber(ROI[p][q] == -ii-1,1,0)		//make a single ROImask for the current ROI
		MatrixOP /o ROIMask=-equal(-ii-1,ROI)+1				//faster code
		Redimension/b/u ROIMask
		
		ImageStats /m=(m) /R=ROIMask image
		
		if(m==1)
			ROIStats[ii][0] = v_avg
			ROIStats[ii][1] = v_min
			ROIStats[ii][2] = v_max
			ROIStats[ii][3] = V_npnts
		else
			ROIStats[ii][0] = v_avg
			ROIStats[ii][1] = v_min
			ROIStats[ii][2] = v_max
			ROIStats[ii][3] = V_npnts
			
			ROIStats[ii][4] = v_sdev
			ROIStats[ii][5] = v_rms
			ROIStats[ii][6] = v_skew
			ROIStats[ii][7] = v_kurt
			ROIStats[ii][8] = v_adev
		endif
	
	EndFor
	
	Return ROIStats
End


/////////////////////////////////////

Function/Wave MultiROIStatsTS(image, ROI, [m])			//calculates ROIstats over a timeseries with a 2D ROI and 3D image stack
	Wave Image, ROI
	Variable m
	
	Variable mismatch

//	Mismatch 

	if (dimsize(image, 0) != dimsize(ROI,0))
		Print "Mismatch in dimension 0"
		mismatch = 1
	elseif (dimsize(image, 1) != dimsize(ROI,1))
		Print "Mismatch in dimension 1"
		mismatch = 1
	else
		mismatch = 0
	endif
	
	If(mismatch)			//just checking...
		DoAlert 0, "Image/ROI mismatch"
		Make/o/n=(1,4) ROIStats = NaN		
		Return ROIStats
	endif
	
	
	If(ParamIsDefault(m))
		m=1
	elseif(!((m == 1) || (m == 2)))
		m = 2
	endif
	
	Variable ROInumber, ii, zDim, jj
	
	ROInumber = -WaveMin(ROI)
	zDim = DimSize(image,2)
	
	if(m==1)
		Make/o/n=(ROInumber,4, zDim) ROIStats
		SetDimLabel 1, 0, Avg, ROIStats
		SetDimLabel 1, 1, Min, ROIStats
		SetDimLabel 1, 2, Max, ROIStats
		SetDimLabel 1, 3, npnts, ROIStats
	else
		Make/o/n=(ROInumber,9, zDim) ROIStats
		SetDimLabel 1, 0, Avg, ROIStats
		SetDimLabel 1, 1, Min, ROIStats
		SetDimLabel 1, 2, Max, ROIStats
		SetDimLabel 1, 3, npnts, ROIStats
		
		SetDimLabel 1, 4, sdev, ROIStats
		SetDimLabel 1, 5, rms, ROIStats
		SetDimLabel 1, 6, skew, ROIStats
		SetDimLabel 1, 7, kurt, ROIStats
		SetDimLabel 1, 8, adev, ROIStats	
	endif
	
	Duplicate/o/free ROI ROImask
	ROImask = 1
	Redimension /b/u ROIMask
	
	
	For(ii=0;ii<ROINumber;ii+=1)
			
	
			MatrixOP /o ROIMask=-equal(-ii-1,ROI)+1				//faster code
			Redimension/b/u ROIMask
	
		
		For(jj=0;jj<zDim;jj+=1)
			Duplicate/o/free/r=[][][jj] image, layer
			
			ImageStats /m=(m) /R=ROIMask layer
			
			if(m==1)
				ROIStats[ii][0][jj] = v_avg
				ROIStats[ii][1][jj] = v_min
				ROIStats[ii][2][jj] = v_max
				ROIStats[ii][3][jj] = V_npnts
			else
				ROIStats[ii][0][jj] = v_avg
				ROIStats[ii][1][jj] = v_min
				ROIStats[ii][2][jj] = v_max
				ROIStats[ii][3][jj] = V_npnts
				
				ROIStats[ii][4][jj] = v_sdev
				ROIStats[ii][5][jj] = v_rms
				ROIStats[ii][6][jj] = v_skew
				ROIStats[ii][7][jj] = v_kurt
				ROIStats[ii][8][jj] = v_adev
			endif
		
		
		EndFor
	EndFor
	
	Return ROIStats
End


/////////////////////////////////////

Function/Wave MultiROIStatsTS3D(image, ROI, [m])			//calculates ROIstats over a timeseries with 3D ROIs
 	Wave Image, ROI
	Variable m
	
	Variable mismatch

	if (dimsize(image, 0) != dimsize(ROI,0))
		Print "Mismatch in dimension 0"
		mismatch = 1
	elseif (dimsize(image, 1) != dimsize(ROI,1))
		Print "Mismatch in dimension 1"
		mismatch = 1
	elseif (dimsize(image, 2) != dimsize(ROI,2))
		Print "Mismatch in dimension 2"
		mismatch = 1
	else
		mismatch = 0
	endif
	
	If(mismatch)			//just checking...
		DoAlert 0, "Image/ROI mismatch"
		Make/o/n=(1,4) ROIStats = NaN		
		Return ROIStats
	endif
	
	
	If(ParamIsDefault(m))
		m=1
	elseif(!((m == 1) || (m == 2)))
		m = 2
	endif
	
	Variable ROInumber, ii, zDim, jj
	
	ROInumber = -WaveMin(ROI)
	zDim = DimSize(image,2)
	
	if(m==1)
		Make/o/n=(ROInumber,4, zDim) ROIStats
		SetDimLabel 1, 0, Avg, ROIStats
		SetDimLabel 1, 1, Min, ROIStats
		SetDimLabel 1, 2, Max, ROIStats
		SetDimLabel 1, 3, npnts, ROIStats
	else
		Make/o/n=(ROInumber,9, zDim) ROIStats
		SetDimLabel 1, 0, Avg, ROIStats
		SetDimLabel 1, 1, Min, ROIStats
		SetDimLabel 1, 2, Max, ROIStats
		SetDimLabel 1, 3, npnts, ROIStats
		
		SetDimLabel 1, 4, sdev, ROIStats
		SetDimLabel 1, 5, rms, ROIStats
		SetDimLabel 1, 6, skew, ROIStats
		SetDimLabel 1, 7, kurt, ROIStats
		SetDimLabel 1, 8, adev, ROIStats	
	endif
	
		Duplicate/o/free ROI ROImask
		FastOP ROImask = 1
		Redimension /b/u ROIMask
	
	
	For(jj=0;jj<zDim;jj+=1)
		
		Duplicate/o/free/r=[][][jj] image, layer
		Duplicate/o/free/r=[][][jj] ROI, ROILayer

		For(ii=0;ii<ROINumber;ii+=1)
	

			MatrixOP /o ROIMask=-equal(-ii-1,ROILayer)+1				//faster code
			Redimension/b/u ROIMask

			ImageStats /m=(m) /R=ROIMask layer
			
			if(m==1)
				ROIStats[ii][0][jj] = v_avg
				ROIStats[ii][1][jj] = v_min
				ROIStats[ii][2][jj] = v_max
				ROIStats[ii][3][jj] = V_npnts
			else
				ROIStats[ii][0][jj] = v_avg
				ROIStats[ii][1][jj] = v_min
				ROIStats[ii][2][jj] = v_max
				ROIStats[ii][3][jj] = V_npnts
				
				ROIStats[ii][4][jj] = v_sdev
				ROIStats[ii][5][jj] = v_rms
				ROIStats[ii][6][jj] = v_skew
				ROIStats[ii][7][jj] = v_kurt
				ROIStats[ii][8][jj] = v_adev
			endif
		
		
		EndFor
	EndFor
	
	Return ROIStats
End





///////////////////////////////////////////////////////////

Function/wave MorphROI(ROI, type)
	wave ROI
	variable type		// 0=Erosion, 1=Dilation, 2=Closing, 3=Opening
	
	
	MatrixOP/o/free binary=-1*(ROI*(-equal(ROI,1)+1))
	
	redimension/b/u binary
	
	switch(type)
		
		case 0:
			ImageMorphology/o Dilation binary //inverted, because binary is inverted relative to ROI
		break
			
		case 1:
			ImageMorphology/o Erosion binary 
		break
		
		case 2:
			ImageMorphology/o Opening binary 
		break 
		
		case 3:
			ImageMorphology/o Closing binary 
		break
		
		default:
			duplicate/o ROI w_MorphROI
			return w_MorphROI
	endswitch
	
	
	MatrixOP /o w_MorphROI=equal(binary,0)-binary
	redimension/s w_MorphROI
	CopyScales ROI, w_MorphROI
	
	return w_MorphROI
end

///////////////////////////////////////////////////////////

Function PixelROIs(ROIWave)					//Calculates a new ROIwave, in which every ROI pixel has a unique negative number
	Wave ROIWave
	
	
	Variable ii, nROIpx, xDim, yDim, zDim, npx, counter
	
	MatrixOP/o w_count = greater (0, ROIWave)		//negative pixels
	redimension/s w_count
	nROIpx = sum(w_count)										//number of ROI pixels
	
	xDim = DimSize(ROIwave, 0)
	yDim = DimSize(ROIwave, 1)
	zDim = DimSize(ROIwave, 2)
	counter = -1
	
	if(zDim>0)
		npx = xDim*yDim*zDim	
	else
		npx = xDim*yDim											//number of all pixels
	endif
	
	Duplicate/o/free ROIwave, w_calc
	
	redimension/n=(npx) w_calc, w_count							//linearize
	
	MultiThread w_calc = PixelROIs_Worker(w_count, p)
	
	if(zDim>0)
		Redimension/n=(xDim,yDim,zDim) w_calc
	else
		Redimension/n=(xDim,yDim) w_calc
	endif
	
	Duplicate/o w_calc, pxROI
	
	CopyScales /P ROIWave, pxROI
	
End


Threadsafe Static Function PixelROIs_Worker(w_count, ii)
	Wave w_count
	variable ii
	
	Variable pnt1, pnt2, Result
	
	pnt1=pnt2x(w_count,0)
	pnt2=pnt2x(w_count,ii)

	
	if(w_count[ii]==0)
		Result = 1
	Else	
		Result = -Sum(w_count, pnt1, pnt2)-1
	Endif
	

	Return Result
End